var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _GossipLog_started, _GossipLog_messages, _GossipLog_services, _GossipLog_pubsub;
import { EventEmitter } from "@libp2p/interface/events";
import { GossipSub } from "@chainsafe/libp2p-gossipsub";
import { logger } from "@libp2p/logger";
import { bytesToHex } from "@noble/hashes/utils";
import openMessageLog from "#store";
import { decodeId } from "./schema.js";
import { SyncService } from "./SyncService.js";
import { assert, nsidPattern } from "./utils.js";
function extractGossipSub(components) {
    const { pubsub } = components;
    assert(pubsub !== undefined, "pubsub service not found");
    assert(pubsub instanceof GossipSub);
    return pubsub;
}
export class GossipLog extends EventEmitter {
    constructor(components, init) {
        super();
        this.components = components;
        this.log = logger(`canvas:gossiplog`);
        _GossipLog_started.set(this, false);
        _GossipLog_messages.set(this, new Map());
        _GossipLog_services.set(this, new Map());
        _GossipLog_pubsub.set(this, void 0);
        this.handleMessage = async ({ detail: msg }) => {
            const { topic, data } = msg;
            const messages = __classPrivateFieldGet(this, _GossipLog_messages, "f").get(topic);
            if (messages === undefined) {
                return;
            }
            const [key, signature, message] = messages.decode(data);
            const id = decodeId(key);
            this.log("received message %s via gossipsub", id);
            const { result, root } = await messages.insert(signature, message);
            this.log("applied message %s and got result %o", id, result);
            this.log("committed new root %s", bytesToHex(root.hash));
        };
        this.location = init.location ?? null;
        this.sync = init.sync ?? true;
        __classPrivateFieldSet(this, _GossipLog_pubsub, extractGossipSub(components), "f");
    }
    isStarted() {
        return __classPrivateFieldGet(this, _GossipLog_started, "f");
    }
    async start() {
        this.log("starting");
        __classPrivateFieldGet(this, _GossipLog_pubsub, "f").addEventListener("message", this.handleMessage);
        __classPrivateFieldSet(this, _GossipLog_started, true, "f");
    }
    async afterStart() {
        for (const service of __classPrivateFieldGet(this, _GossipLog_services, "f").values()) {
            await service.start();
        }
    }
    async beforeStop() {
        await Promise.all(Array.from(__classPrivateFieldGet(this, _GossipLog_services, "f")).map(async ([topic, service]) => {
            await service.stop();
            __classPrivateFieldGet(this, _GossipLog_services, "f").delete(topic);
        }));
    }
    async stop() {
        this.log("stopping");
        __classPrivateFieldGet(this, _GossipLog_pubsub, "f").removeEventListener("message", this.handleMessage);
        for (const messages of __classPrivateFieldGet(this, _GossipLog_messages, "f").values()) {
            await messages.close();
        }
        __classPrivateFieldGet(this, _GossipLog_messages, "f").clear();
        __classPrivateFieldGet(this, _GossipLog_services, "f").clear();
        __classPrivateFieldSet(this, _GossipLog_started, false, "f");
    }
    async subscribe(topic, init) {
        const { validate, apply, signatures, sequencing, ...syncOptions } = init;
        assert(nsidPattern.test(topic), "invalid topic (must match [a-zA-Z0-9\\.\\-]+)");
        this.log("subscribing to %s", topic);
        const messages = await openMessageLog({ location: this.location, topic, apply, validate, signatures, sequencing });
        __classPrivateFieldGet(this, _GossipLog_messages, "f").set(topic, messages);
        messages.addEventListener("message", ({ detail: { id, signature, message, result } }) => this.safeDispatchEvent("message", { detail: { topic, id, signature, message, result } }));
        messages.addEventListener("commit", ({ detail: { root } }) => this.safeDispatchEvent("commit", { detail: { topic, root } }));
        messages.addEventListener("sync", ({ detail: { peerId } }) => this.safeDispatchEvent("sync", { detail: { topic, peerId } }));
        if (this.sync) {
            const service = new SyncService(this.components, messages, syncOptions);
            __classPrivateFieldGet(this, _GossipLog_services, "f").set(topic, service);
            if (__classPrivateFieldGet(this, _GossipLog_started, "f")) {
                await service.start();
            }
        }
        if (__classPrivateFieldGet(this, _GossipLog_started, "f")) {
            __classPrivateFieldGet(this, _GossipLog_pubsub, "f").subscribe(topic);
        }
    }
    async unsubscribe(topic) {
        this.log("unsubscribing from %s", topic);
        if (__classPrivateFieldGet(this, _GossipLog_started, "f")) {
            __classPrivateFieldGet(this, _GossipLog_pubsub, "f").unsubscribe(topic);
        }
        const service = __classPrivateFieldGet(this, _GossipLog_services, "f").get(topic);
        if (service !== undefined) {
            await service.stop().finally(() => __classPrivateFieldGet(this, _GossipLog_services, "f").delete(topic));
        }
        const messages = __classPrivateFieldGet(this, _GossipLog_messages, "f").get(topic);
        if (messages !== undefined) {
            await messages.close().finally(() => __classPrivateFieldGet(this, _GossipLog_messages, "f").delete(topic));
        }
    }
    async publish(topic, payload, options = {}) {
        const messages = __classPrivateFieldGet(this, _GossipLog_messages, "f").get(topic);
        assert(messages !== undefined, "no subscription for topic");
        const { id, value, result } = await messages.append(payload, options);
        if (__classPrivateFieldGet(this, _GossipLog_started, "f")) {
            const recipients = __classPrivateFieldGet(this, _GossipLog_pubsub, "f").publish(topic, value).then(({ recipients }) => {
                this.log("published message %s to %d recipients %O", id, recipients.length, recipients);
                return recipients;
            }, (err) => {
                this.log.error("failed to publish event: %O", err);
                return [];
            });
            return { id, result, recipients };
        }
        else {
            return { id, result, recipients: Promise.resolve([]) };
        }
    }
    async apply(topic, signature, message) {
        const messages = __classPrivateFieldGet(this, _GossipLog_messages, "f").get(topic);
        assert(messages !== undefined, "topic not found");
        const { id, result, value } = await messages.insert(signature, message);
        if (__classPrivateFieldGet(this, _GossipLog_started, "f")) {
            const recipients = __classPrivateFieldGet(this, _GossipLog_pubsub, "f").publish(topic, value).then(({ recipients }) => {
                this.log("published message %s to %d recipients %O", id, recipients.length, recipients);
                return recipients;
            }, (err) => {
                this.log.error("failed to publish event: %O", err);
                return [];
            });
            return { id, result, recipients };
        }
        else {
            return { id, result, recipients: Promise.resolve([]) };
        }
    }
    getTopics() {
        return [...__classPrivateFieldGet(this, _GossipLog_messages, "f").keys()];
    }
    async *iterate(topic, lowerBound = null, upperBound = null, options = {}) {
        const messages = __classPrivateFieldGet(this, _GossipLog_messages, "f").get(topic);
        assert(messages !== undefined, "topic not found");
        yield* messages.iterate(lowerBound, upperBound, options);
    }
    async getClock(topic) {
        const messages = __classPrivateFieldGet(this, _GossipLog_messages, "f").get(topic);
        assert(messages !== undefined, "topic not found");
        return await messages.getClock();
    }
}
_GossipLog_started = new WeakMap(), _GossipLog_messages = new WeakMap(), _GossipLog_services = new WeakMap(), _GossipLog_pubsub = new WeakMap();
export const gossiplog = (init) => (components) => new GossipLog(components, init);
