import type { PeerId } from "@libp2p/interface/peer-id";
import type { Connection } from "@libp2p/interface/connection";
import type { ConnectionManager } from "@libp2p/interface-internal/connection-manager";
import type { Registrar } from "@libp2p/interface-internal/registrar";
import type { Startable } from "@libp2p/interface/startable";
import { AbstractMessageLog } from "./store/AbstractMessageLog.js";
export interface Topology {
    min?: number;
    max?: number;
    onConnect?: (peerId: PeerId, conn: Connection) => void;
    onDisconnect?: (peerId: PeerId) => void;
}
export interface SyncOptions {
    minConnections?: number;
    maxConnections?: number;
    maxInboundStreams?: number;
    maxOutboundStreams?: number;
}
export interface SyncServiceComponents {
    peerId: PeerId;
    registrar: Registrar;
    connectionManager: ConnectionManager;
}
export declare class SyncService<Payload = unknown, Result = void> implements Startable {
    #private;
    private readonly components;
    private readonly messages;
    private readonly protocol;
    private readonly topology;
    private readonly topologyPeers;
    private readonly maxInboundStreams;
    private readonly maxOutboundStreams;
    private readonly minConnections;
    private readonly maxConnections;
    private readonly syncQueue;
    private readonly syncQueuePeers;
    private readonly syncHistory;
    private readonly log;
    constructor(components: SyncServiceComponents, messages: AbstractMessageLog<Payload, Result>, options: SyncOptions);
    isStarted(): boolean;
    get topic(): string;
    start(): Promise<void>;
    stop(): Promise<void>;
    private handleIncomingStream;
    private scheduleSync;
    private wait;
    private sync;
    private dial;
}
