import { anySignal } from "any-signal";
import { lessThan } from "@canvas-js/okra";
export const nsidPattern = /^[a-z](?:-*[a-z0-9])*(?:\.[a-z](?:-*[a-z0-9])*)*$/;
export function sortPair(a, b) {
    if (lessThan(a.multihash.digest, b.multihash.digest)) {
        return [a, b];
    }
    else {
        return [b, a];
    }
}
export function assert(condition, message) {
    if (!condition) {
        throw new Error(message ?? "assertion failed");
    }
}
export function signalInvalidType(type) {
    console.error(type);
    throw new TypeError("internal error: invalid type");
}
export async function wait(interval, options) {
    if (options.signal.aborted) {
        return;
    }
    const signal = anySignal([AbortSignal.timeout(interval), options.signal]);
    await new Promise((resolve) => {
        signal.addEventListener("abort", () => resolve());
    }).finally(() => signal.clear());
}
export class CacheMap extends Map {
    constructor(capacity, entries) {
        super();
        this.capacity = capacity;
    }
    set(key, value) {
        super.set(key, value);
        for (const key of this.keys()) {
            if (this.size > this.capacity) {
                this.delete(key);
            }
            else {
                break;
            }
        }
        return this;
    }
}
export function shuffle(array) {
    for (let i = array.length - 1; i > 0; i--) {
        const j = Math.floor(Math.random() * (i + 1));
        [array[i], array[j]] = [array[j], array[i]];
    }
}
